package main

import (
	"context"
	"strconv"

	"github.com/camunda-cloud/zeebe/clients/go/pkg/entities"
	"gitlab.mashroom.com/mashroom/backend/common/api/grpcerr"
	"gitlab.mashroom.com/mashroom/backend/common/generic/bpms"
	"gitlab.mashroom.com/mashroom/backend/common/generic/env"
	messaging "gitlab.mashroom.com/mashroom/backend/messaging/api"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/service"
)

var (
	envVibrantEmail = env.ResolveString("VIBRANT_EMAIL", "orders@mashroom.com")
)

type sendPlatformRequest struct {
	notification.SendPlatformRequest
	Process    string `json:"process"`
	Property   string `json:"property"`
	User       string `json:"user"`
	Invitation string `json:"invitation"`
	Offer      string `json:"offer"`
	Viewing    string `json:"viewing"`
	ErrorKind  string `json:"error_kind"`
	ErrorValue string `json:"error_value"`
}

func serveBPMS(srv service.Service) {
	_, err := bpms.ServeBpms("notification-service", bpms.MapHandler(map[string]bpms.Registrator{
		"send-platform": func() (request interface{}, processor bpms.Processor) {
			req := &sendPlatformRequest{}
			return req,
				func(
					ctx context.Context,
					job entities.Job,
				) (reply interface{}, enhance map[string]interface{}, err error) {
					if req.User != "" {
						req.Embeds = append(req.Embeds, &notification.Embed{
							EmbedType:  int32(messaging.EmbedType_EMBED_TYPE_USER),
							EmbedValue: req.User,
						})
					}
					if req.Property != "" {
						req.Embeds = append(req.Embeds, &notification.Embed{
							EmbedType:  int32(messaging.EmbedType_EMBED_TYPE_PROPERTY),
							EmbedValue: req.Property,
						})
					}
					if req.ErrorKind != "" {
						req.Embeds = append(req.Embeds, &notification.Embed{
							EmbedType:  int32(messaging.EmbedType_EMBED_TYPE_ERRORS),
							EmbedValue: req.ErrorKind + ":" + req.ErrorValue,
						})
					}
					if req.Invitation != "" {
						req.Embeds = append(req.Embeds, &notification.Embed{
							EmbedType:  int32(messaging.EmbedType_EMBED_TYPE_INVITATION),
							EmbedValue: req.Invitation,
						})
					}
					if req.Offer != "" {
						req.Embeds = append(req.Embeds, &notification.Embed{
							EmbedType:  int32(messaging.EmbedType_EMBED_TYPE_OFFER),
							EmbedValue: req.Offer,
						})
					}
					if req.Viewing != "" {
						req.Embeds = append(req.Embeds, &notification.Embed{
							EmbedType:  int32(messaging.EmbedType_EMBED_TYPE_VIEWING),
							EmbedValue: req.Viewing,
						})
					}
					req.Embeds = append(req.Embeds, &notification.Embed{
						EmbedType:  int32(messaging.EmbedType_EMBED_TYPE_PROCESS),
						EmbedValue: strconv.FormatInt(job.GetProcessInstanceKey(), 10),
					})
					if req.GetBody() == "A new offer submitted" {
						return &notification.SendPlatformResponse{}, nil, nil
					}
					if req.GetBody() == "Offer was withdrawn" {
						return &notification.SendPlatformResponse{}, nil, nil
					}

					req.Event = req.GetEvent()

					resp, err := srv.SendPlatform(context.Background(), &req.SendPlatformRequest)
					return resp, nil, err
				}
		},
		"send-vibrant": func() (request interface{}, processor bpms.Processor) {
			req := &struct {
				VibrantFileID string `json:"vibrantFileId"`
			}{}
			return req,
				func(
					ctx context.Context,
					job entities.Job,
				) (reply interface{}, enhance map[string]interface{}, err error) {
					if req.VibrantFileID == "" {
						return nil, nil, grpcerr.New(grpcerr.InvalidData, "can't notify Vibrant, file ID is empty")
					}
					resp, err := srv.SendEmail(ctx, &notification.SendEmailRequest{
						To:      envVibrantEmail,
						Subject: "New vibrant service ordered through mashroom",
						Body: "Please find attached a csv file with the details of the order" +
							" placed on the mashroom platform.",
						Attachments: []string{req.VibrantFileID},
					})
					return resp, nil, err
				}
		},
	}))
	if err != nil {
		panic(err)
	}
}
