package main

import (
	"context"
	"fmt"
	"net"
	"net/url"
	"os"
	"strings"
	"time"

	"github.com/go-redis/redis/v8"
	"github.com/jmoiron/sqlx"
	"github.com/sfreiberg/gotwilio"
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/common/api/health"
	"gitlab.mashroom.com/mashroom/backend/common/generic/addr"
	"gitlab.mashroom.com/mashroom/backend/common/generic/env"
	"gitlab.mashroom.com/mashroom/backend/common/generic/grpc"
	logger "gitlab.mashroom.com/mashroom/backend/common/generic/log"
	"gitlab.mashroom.com/mashroom/backend/common/generic/metrics"
	document "gitlab.mashroom.com/mashroom/backend/document/api"
	invitation "gitlab.mashroom.com/mashroom/backend/invitation/api"
	maintenance "gitlab.mashroom.com/mashroom/backend/maintenance/api"
	messaging "gitlab.mashroom.com/mashroom/backend/messaging/api"
	mortgage "gitlab.mashroom.com/mashroom/backend/mortgage/api"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/embed"
	embedCommon "gitlab.mashroom.com/mashroom/backend/notification/internal/embed/common"
	embedDocument "gitlab.mashroom.com/mashroom/backend/notification/internal/embed/document"
	embedInvitation "gitlab.mashroom.com/mashroom/backend/notification/internal/embed/invitation"
	embedMaintenance "gitlab.mashroom.com/mashroom/backend/notification/internal/embed/maintenance"
	embedMortgage "gitlab.mashroom.com/mashroom/backend/notification/internal/embed/mortgage"
	embedOffer "gitlab.mashroom.com/mashroom/backend/notification/internal/embed/offer"
	embedProperty "gitlab.mashroom.com/mashroom/backend/notification/internal/embed/property"
	embedTenancy "gitlab.mashroom.com/mashroom/backend/notification/internal/embed/tenancy"
	embedTime "gitlab.mashroom.com/mashroom/backend/notification/internal/embed/time"
	embedUser "gitlab.mashroom.com/mashroom/backend/notification/internal/embed/user"
	embedViewing "gitlab.mashroom.com/mashroom/backend/notification/internal/embed/viewing"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/handlers"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/handlers/platform"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/handlers/sendgrid"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/handlers/smtp"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/handlers/twilio"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/model/postgres"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/service"
	redisRepository "gitlab.mashroom.com/mashroom/backend/notification/internal/task/redis"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/template/root"
	"gitlab.mashroom.com/mashroom/backend/notification/version"
	offer "gitlab.mashroom.com/mashroom/backend/offer/api"
	property "gitlab.mashroom.com/mashroom/backend/property/api"
)

var (
	connectionMessagingService = messaging.NewMessagingServiceClient(grpc.Dial(
		env.ResolveString("MESSAGING_SERVICE_URL", "mashroom-messaging:10000"),
		"Messaging",
	))

	connectionPropertyService = property.NewPropertyServiceClient(grpc.Dial(
		env.ResolveString("PROPERTY_SERVICE_URL", "mashroom-property:10000"),
		"Property",
	))

	connectionAccountService = account.NewAccountServiceClient(grpc.Dial(
		env.ResolveString("ACCOUNT_SERVICE_URL", "mashroom-account:10000"),
		"Account",
	))

	connectionInvitationService = invitation.NewInvitationServiceClient(grpc.Dial(
		env.ResolveString("INVITATION_SERVICE_URL", "mashroom-invitation:10000"),
		"Invitation",
	))

	connectionMaintenanceService = maintenance.NewMaintenanceServiceClient(grpc.Dial(
		env.ResolveString("MAINTENANCE_SERVICE_URL", "mashroom-maintenance:10000"),
		"Maintenance",
	))

	connectionOfferService = offer.NewOfferServiceClient(grpc.Dial(
		env.ResolveString("OFFER_SERVICE_URL", "mashroom-offer:10000"),
		"Offer",
	))

	connectionDocumentService = document.NewDocumentServiceClient(grpc.Dial(
		env.ResolveString("DOCUMENT_SERVICE_URL", "mashroom-document:10000"),
		"Document",
	))

	connectionMortgageService = mortgage.NewMortgageServiceClient(grpc.Dial(
		env.ResolveString("MORTGAGE_SERVICE_URL", "mashroom-mortgage:10000"),
		"Mortgage",
	))
)

var (
	envSiteURL          = env.ResolveString("SITE_URL", "https://dev.env.mashroom.com")
	envAdminSiteURL     = env.ResolveString("ADMIN_SITE_URL", "")
	envSupportEmail     = env.ResolveString("SUPPORT_EMAIL", "support@mashroom.com")
	envProduction       = env.ResolveBool("PRODUCTION", false)
	envRedisAddress     = env.ResolveString("REDIS_ADDR", "mashroom-notification-redis:6379")
	envDequeuePeriod    = env.ResolveDuration("DEQUEUE_PERIOD", time.Second)
	envConsumerID       = env.ResolveString("CONSUMER_ID", "consumer-global")
	envResolverCacheTTL = env.ResolveDuration("RESOLVER_CACHE_TTL", time.Second*15)
)

var (
	envTwilioAuthToken       = env.ResolveString("TWILIO_AUTH_TOKEN", "")
	envTwilioAccountSID      = env.ResolveString("TWILIO_ACCOUNT_SID", "")
	envTwilioServiceSID      = env.ResolveString("TWILIO_MESSAGING_SERVICE_SID", "")
	envTwilioServiceCallback = env.ResolveString("TWILIO_SERVICE_CALLBACK", "")
	envTwilioApplicationSID  = env.ResolveString("TWILIO_APPLICATION_SID", "")
)

var (
	envSendgridEnable        = env.ResolveBool("SENDGRID_ENABLE", false)
	envSendgridLeadsListName = env.ResolveString("SENDGRID_LEADS_LIST_NAME", "Leads")
	envSendgridKey           = env.ResolveString(
		"SENDGRID_KEY",
		"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",
	)
)

var (
	envEmailFromAddress = env.ResolveString("FROM_ADDRESS", "hello@mashroom.com")
	envEmailFromName    = env.ResolveString("FROM_NAME", "Mashroom")
	envSMTPServer       = env.ResolveString("SMTP_SERVER", "localhost:25")
)

func makeEmbedRoot() *embed.Root {
	invitationResolver := embedCommon.NewCache(&embedInvitation.Resolver{
		Invitation: connectionInvitationService,
		Account:    connectionAccountService,
		SiteURL:    envSiteURL,
	}, envResolverCacheTTL)

	userResolver := embedCommon.NewCache(&embedUser.Resolver{
		Account: connectionAccountService,
	}, envResolverCacheTTL)

	propertyResolver := embedCommon.NewCache(&embedProperty.Resolver{
		Property:        connectionPropertyService,
		AccountResolver: userResolver,
		SiteURL:         envSiteURL,
	}, envResolverCacheTTL)

	maintenanceResolver := embedCommon.NewCache(&embedMaintenance.Resolver{
		Maintenance: connectionMaintenanceService,
		SiteURL:     envSiteURL,
	}, envResolverCacheTTL)

	offerResolver := embedCommon.NewCache(&embedOffer.Resolver{
		Offer: connectionOfferService,
	}, envResolverCacheTTL)

	viewingResolver := embedCommon.NewCache(&embedViewing.Resolver{
		Property: connectionPropertyService,
	}, envResolverCacheTTL)

	tenancyResolver := embedCommon.NewCache(&embedTenancy.Resolver{
		Property: connectionPropertyService,
	}, envResolverCacheTTL)

	documentResolver := embedCommon.NewCache(&embedDocument.Resolver{
		Document: connectionDocumentService,
	}, envResolverCacheTTL)

	leadResolver := embedCommon.NewCache(&embedMortgage.Lead{
		Mortgage: connectionMortgageService,
		Account:  connectionAccountService,
		SiteURL:  envSiteURL,
	}, envResolverCacheTTL)

	unixResolver := &embedTime.Unix{
		Format: "2006-01-02 15:04:05",
	}

	firstResolver := &embedCommon.First{}

	allResolver := &embedCommon.All{}

	jsonResolver := &embedCommon.JSON{}

	registry := embed.Registry{
		messaging.EmbedType_EMBED_TYPE_INVITATION:    embed.NewEntry(invitationResolver, "invitation"),
		messaging.EmbedType_EMBED_TYPE_USER:          embed.NewEntry(userResolver, "user"),
		messaging.EmbedType_EMBED_TYPE_PROPERTY:      embed.NewEntry(propertyResolver, "property"),
		messaging.EmbedType_EMBED_TYPE_MAINTENANCE:   embed.NewEntry(maintenanceResolver, "maintenance"),
		messaging.EmbedType_EMBED_TYPE_VIEWING:       embed.NewEntry(viewingResolver, "viewing"),
		messaging.EmbedType_EMBED_TYPE_OFFER:         embed.NewEntry(offerResolver, "offer"),
		messaging.EmbedType_EMBED_TYPE_TENANCY:       embed.NewEntry(tenancyResolver, "tenancy"),
		messaging.EmbedType_EMBED_TYPE_DOCUMENT:      embed.NewEntry(documentResolver, "document"),
		messaging.EmbedType_EMBED_TYPE_MORTGAGE_LEAD: embed.NewEntry(leadResolver, "lead"),
		messaging.EmbedType_EMBED_TYPE_PAYMENT_COST:  embed.NewEntry(firstResolver, "payment", "cost"),
		messaging.EmbedType_EMBED_TYPE_PAYMENT_ITEM:  embed.NewEntry(firstResolver, "payment", "item"),
		messaging.EmbedType_EMBED_TYPE_PAYMENT_TIME:  embed.NewEntry(unixResolver, "payment", "time"),
		messaging.EmbedType_EMBED_TYPE_TOKEN:         embed.NewEntry(firstResolver, "token"),
		messaging.EmbedType_EMBED_TYPE_SECONDS:       embed.NewEntry(firstResolver, "root", "seconds"),
		messaging.EmbedType_EMBED_TYPE_MINUTES:       embed.NewEntry(firstResolver, "root", "minutes"),
		messaging.EmbedType_EMBED_TYPE_HOURS:         embed.NewEntry(firstResolver, "root", "hours"),
		messaging.EmbedType_EMBED_TYPE_DAYS:          embed.NewEntry(firstResolver, "root", "days"),
		messaging.EmbedType_EMBED_TYPE_MONTHS:        embed.NewEntry(firstResolver, "root", "months"),
		messaging.EmbedType_EMBED_TYPE_YEARS:         embed.NewEntry(firstResolver, "root", "years"),
		messaging.EmbedType_EMBED_TYPE_FILE:          embed.NewEntry(allResolver, "files"),
		messaging.EmbedType_EMBED_TYPE_JSON:          embed.NewEntry(jsonResolver),
	}

	return &embed.Root{
		Registry:       registry,
		RecipientEntry: embed.NewEntry(userResolver, "recipient"),
		StaticValues: []*embed.StaticValue{
			{
				Values: []string{envSiteURL},
				Entry:  embed.NewEntry(firstResolver, "site"),
			},
			{
				Values: []string{envAdminSiteURL},
				Entry:  embed.NewEntry(firstResolver, "adminsite"),
			},
		},
	}
}

func main() {
	if !strings.HasPrefix(envSiteURL, "http") {
		envSiteURL = "https://" + envSiteURL
	}

	purl, err := url.Parse(envSiteURL)
	if err != nil {
		panic(err)
	}

	if envAdminSiteURL == "" {
		purl.Host = "admin." + purl.Host

		envAdminSiteURL = purl.String()
	}

	if strings.HasPrefix(envSiteURL, "https://mashroom.com") {
		envProduction = true
	}

	ctx := context.Background()
	conn := fmt.Sprintf(
		"postgres://%s:%s@%s:%s/%s?sslmode=disable",
		os.Getenv("POSTGRES_USERNAME"),
		os.Getenv("POSTGRES_PASSWORD"),
		os.Getenv("POSTGRES_HOST"),
		os.Getenv("POSTGRES_PORT"),
		os.Getenv("POSTGRES_DATABASE"),
	)

	db := sqlx.MustOpen("postgres", conn)

	repo, err := postgres.NewRepository(db)
	if err != nil {
		panic(err)
	}

	err = repo.Migrate(ctx)
	if err != nil {
		panic(err)
	}

	redisClient := redis.NewClient(&redis.Options{
		Addr:     envRedisAddress,
		Password: "",
		DB:       0,
	})

	queue, err := redisRepository.NewRepository(ctx, &redisRepository.Config{
		RedisClient: redisClient,
		Consumer:    envConsumerID,
		Handlers: []string{
			service.HandlerEmail,
			service.HandlerSMS,
			service.HandlerPlatform,
		},
	})
	if err != nil {
		panic(err)
	}

	handlerSMS := twilio.NewHandler(&twilio.Config{
		Client:         gotwilio.NewTwilioClient(envTwilioAccountSID, envTwilioAuthToken),
		ServiceSID:     envTwilioServiceSID,
		StatusCallback: envTwilioServiceCallback,
		ApplicationSID: envTwilioApplicationSID,
		Site:           envSiteURL,
	})

	handlerPlatform := platform.NewHandler(&platform.Config{
		Messaging: connectionMessagingService,
	})

	var handlerEmail handlers.Handler

	if envSendgridEnable {
		handlerEmail = sendgrid.NewHandler(&sendgrid.Config{
			SendgridKey:     envSendgridKey,
			FromName:        envEmailFromName,
			FromAddress:     envEmailFromAddress,
			Site:            envSiteURL,
			DocumentService: connectionDocumentService,
		})
	} else {
		handlerEmail = smtp.NewHandler(&smtp.Config{
			SMTPServer:      envSMTPServer,
			SMTPAuth:        nil,
			FromName:        envEmailFromName,
			FromAddress:     envEmailFromAddress,
			Site:            envSiteURL,
			DocumentService: connectionDocumentService,
		})
	}

	leadsList, err := resolveSengridListID(envSendgridLeadsListName)
	if err != nil {
		panic(err)
	}

	logger.Info("leads list ID resolved: " + leadsList)

	srv, err := service.NewService(ctx, &service.Config{
		Handlers: service.Handlers{
			Email:    handlerEmail,
			SMS:      handlerSMS,
			Platform: handlerPlatform,
		},
		Repository:        repo,
		Queue:             queue,
		DequeuePeriod:     envDequeuePeriod,
		Embeds:            makeEmbedRoot(),
		Site:              envSiteURL,
		SupportEmail:      envSupportEmail,
		Production:        envProduction,
		Templates:         root.Registry,
		SendgridKey:       envSendgridKey,
		SendgridListLeads: leadsList,
	})
	if err != nil {
		panic(err)
	}

	listener, err := net.Listen("tcp", addr.Grpc)
	if err != nil {
		panic(err)
	}

	grpcServer, err := grpc.NewServer("notification")
	if err != nil {
		panic(err)
	}

	notification.RegisterNotificationServiceServer(grpcServer, srv)

	logger.Infoln("notification started")

	healthServer := version.NewHealthServer()
	health.RegisterHealthServiceServer(grpcServer, healthServer)

	go func() {
		_ = metrics.ConfigureListenAndServe(grpcServer)
	}()

	go func() {
		serveBPMS(srv)
	}()

	err = grpcServer.Serve(listener)
	if err != nil {
		logger.Errorln("gRPC error", err)
	}
}
