// Package embed provides framework for resolving referenced embedded objects
package embed

import (
	"context"
	"errors"
	"fmt"
	"strings"

	messaging "gitlab.mashroom.com/mashroom/backend/messaging/api"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
)

// Values resolved embedded values
type Values map[string]interface{}

// Resolver for embedded value
type Resolver interface {
	Resolve(ctx context.Context, values ...string) (interface{}, error)
}

// StaticValue pair of resolver entry and static values
type StaticValue struct {
	Values []string
	Entry  *Entry
}

// Entry pair of resolver and output path
type Entry struct {
	Path     []string
	Resolver Resolver
}

// NewEntry utility function to create entry
func NewEntry(resolver Resolver, path ...string) *Entry {
	return &Entry{
		Path:     path,
		Resolver: resolver,
	}
}

// Registry registry of embed type-dispatched entries
type Registry map[messaging.EmbedType]*Entry

// Root embed resolver
type Root struct {
	Registry       Registry
	RecipientEntry *Entry
	StaticValues   []*StaticValue
}

// Resolve takes recipient, list of embeds and applies all configured resolvers to provided embeds, yielding
// resolved embed Values
func (reg *Root) Resolve(
	ctx context.Context,
	recipient string,
	recipientContacts *notification.Contacts,
	embeds []*notification.EventEmbed,
) (values Values, rcpt interface{}, err error) {
	values = make(Values)
	embedValues := make(map[messaging.EmbedType][]string)

	for _, e := range embeds {
		embedValues[e.GetEmbedType()] = append(embedValues[e.GetEmbedType()], e.GetEmbedValue())
	}

	ctx = context.WithValue(ctx, recipientKey, recipient)
	ctx = context.WithValue(ctx, embedValuesKey, embedValues)

	var errs []string

	for t, vs := range embedValues {
		entry, ok := reg.Registry[t]
		if !ok {
			continue
		}

		var value interface{}

		value, err = entry.Resolver.Resolve(ctx, vs...)
		if err != nil {
			errs = append(errs, fmt.Sprintf("resolving embed %v(%v): %s", t, vs, err.Error()))
		}

		SetMap(values, value, entry.Path...)
	}

	ctx = context.WithValue(ctx, contactsKey, recipientContacts)

	if reg.RecipientEntry != nil {
		rcpt, err = reg.RecipientEntry.Resolver.Resolve(ctx, recipient)
		if err != nil {
			errs = append(errs, fmt.Sprintf("resolving recipient %s: %s", recipient, err.Error()))
		}

		SetMap(values, rcpt, reg.RecipientEntry.Path...)
	}

	for _, s := range reg.StaticValues {
		var value interface{}

		value, err = s.Entry.Resolver.Resolve(ctx, s.Values...)
		if err != nil {
			errs = append(
				errs,
				fmt.Sprintf("resolving static entry(%v) %v: %s", s.Entry.Path, s.Values, err.Error()),
			)
		}

		SetMap(values, value, s.Entry.Path...)
	}

	if len(errs) > 0 {
		err = errors.New(strings.Join(errs, ", "))
	}

	return
}
