// Package common provides common/generic embed resolvers
package common

import (
	"context"
	"encoding/json"
	"strings"
	"time"

	"gitlab.mashroom.com/mashroom/backend/notification/internal/embed"
	"zgo.at/zcache"
)

// First resolver returns first value of embeds values
type First struct{}

// Resolve implementation
func (First) Resolve(ctx context.Context, values ...string) (interface{}, error) {
	if len(values) == 0 {
		return nil, nil
	}

	return values[0], nil
}

// All resolver returns all embed values as a slice
type All struct{}

// Resolve implementation
func (All) Resolve(ctx context.Context, values ...string) (interface{}, error) {
	return values, nil
}

// Cache wraps resolver with cache
type Cache struct {
	Resolver embed.Resolver
	Cache    *zcache.Cache
}

// NewCache utility function to wrap resolver with timed cache
func NewCache(resolver embed.Resolver, ttl time.Duration) *Cache {
	return &Cache{
		Resolver: resolver,
		Cache:    zcache.New(ttl, ttl*2),
	}
}

// Resolve implementation
func (c *Cache) Resolve(ctx context.Context, values ...string) (interface{}, error) {
	key := strings.Join(values, "-")

	if len(key) == 0 {
		return c.Resolver.Resolve(ctx, values...)
	}

	v, ok := c.Cache.Get(key)
	if ok {
		return v, nil
	}

	v, err := c.Resolver.Resolve(ctx, values...)
	if err != nil {
		return v, err
	}

	c.Cache.SetDefault(key, v)

	return v, err
}

// JSON resolver returns merged json values
type JSON struct{}

// Resolve implementation
func (JSON) Resolve(ctx context.Context, values ...string) (interface{}, error) {
	if len(values) == 0 {
		return nil, nil
	}

	res := make(map[string]interface{})

	for _, v := range values {
		var m map[string]interface{}

		err := json.Unmarshal(([]byte)(v), &m)
		if err != nil {
			continue
		}

		embed.MergeMaps(res, m)
	}

	return res, nil
}
