package embed

import (
	"fmt"
	"reflect"
	"strings"

	"gitlab.mashroom.com/mashroom/backend/common/generic/bind"
)

// MergeMaps combines two maps preferring non-nil values from both and target values when both are present
func MergeMaps(source, target map[string]interface{}) {
	for k, v := range target {
		SetMap(source, v, k)
	}
}

// GetMap returns a value from nested maps
func GetMap(v interface{}, path ...string) interface{} {
	for _, p := range path {
		if v == nil {
			return nil
		}

		m, ok := v.(map[string]interface{})
		if !ok {
			return nil
		}

		v, ok = m[p]
		if !ok {
			return nil
		}
	}

	return v
}

// GetMapString returns a string value from nested maps
func GetMapString(v interface{}, path ...string) string {
	vs := GetMap(v, path...)

	if vs == nil {
		return ""
	}

	return fmt.Sprint(vs)
}

// SetMap sets a path in interface map to a value, creating new interface maps if necessary and merging existing maps
// if possible, preferring first non-nil, then target values
func SetMap(v map[string]interface{}, value interface{}, path ...string) {
	if len(path) == 0 {
		tm, ok := value.(map[string]interface{})
		if ok {
			MergeMaps(v, tm)
		}

		return
	}

	prefix, last := path[:len(path)-1], path[len(path)-1]

	for _, p := range prefix {
		nested, ok := v[p]
		if !ok {
			nested = make(map[string]interface{})
			v[p] = nested
		}

		nestedmap, ok := nested.(map[string]interface{})
		if !ok {
			nestedmap = make(map[string]interface{})
			v[p] = nestedmap
		}

		v = nestedmap
	}

	if v[last] != nil {
		if value == nil {
			return
		}

		tmap, tok := value.(map[string]interface{})
		smap, sok := v[last].(map[string]interface{})

		if tok && sok {
			MergeMaps(smap, tmap)

			return
		}
	}

	v[last] = value
}

// BindMap converts given protobuf structure to an interface map
func BindMap(v interface{}) map[string]interface{} {
	rval := reflect.ValueOf(v)
	rtyp := bind.GetType(rval.Type())
	rmap, _ := rtyp.GetValue(rval, &bind.Config{})

	val, ok := rmap.Interface().(map[string]interface{})
	if !ok {
		return map[string]interface{}{}
	}

	return val
}

// JoinNonempty joins all nonempty strings using provided separator
func JoinNonempty(sep string, values ...string) string {
	var nonempty []string

	for _, v := range values {
		trimmed := strings.TrimSpace(v)

		if len(trimmed) > 0 {
			nonempty = append(nonempty, trimmed)
		}
	}

	return strings.Join(nonempty, sep)
}
