package sendgrid

import (
	"bytes"
	"context"
	"encoding/base64"
	"encoding/json"
	"time"

	"github.com/sendgrid/sendgrid-go"
	"github.com/sendgrid/sendgrid-go/helpers/mail"
	"gitlab.mashroom.com/mashroom/backend/common/api/grpcerr"
	document "gitlab.mashroom.com/mashroom/backend/document/api"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/handlers/assets"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/task"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/template"
)

type handlerImpl struct {
	config Config
}

var contentTypes = map[bool]string{
	true:  "text/html",
	false: "text/plain",
}

func (handler *handlerImpl) Run(ctx context.Context, instance *task.Task) error {
	var t task.Email

	err := json.Unmarshal(instance.Data, &t)
	if err != nil {
		return err
	}

	email := mail.NewV3Mail()
	email.Subject = t.Subject

	email.SetFrom(mail.NewEmail(handler.config.FromName, handler.config.FromAddress))

	var tos []*mail.Email

	for _, to := range t.To {
		tos = append(tos, mail.NewEmail(to.Name, to.Address))
	}

	if t.ReplyTo != nil {
		email.SetReplyTo(mail.NewEmail(t.ReplyTo.Name, t.ReplyTo.Address))
	}

	p := mail.NewPersonalization()
	p.AddTos(tos...)

	email.AddPersonalizations(p)

	body := t.Body
	tmplid := t.Data[template.SendgridTemplateID]

	if !t.Raw && tmplid == "" {
		body, err = assets.RenderEmailTemplate(handler.config.Site, t.Subject, body)
		if err != nil {
			return err
		}
	}

	if tmplid == "" && body != "" {
		email.AddContent(mail.NewContent(contentTypes[t.HTML], body))
	} else {
		email.SetTemplateID(tmplid)
		err = json.Unmarshal(t.Embeds, &p.DynamicTemplateData)
		if err != nil {
			return err
		}
	}

	for _, f := range t.Attachments {
		b64 := &bytes.Buffer{}
		encoder := base64.NewEncoder(base64.StdEncoding, b64)

		var info *document.File

		info, err = document.DownloadFile(ctx, handler.config.DocumentService, f, document.ImageSize_ORIGINAL, encoder)
		if err != nil {
			return err
		}

		err = encoder.Close()
		if err != nil {
			return err
		}

		a := mail.NewAttachment()

		a.SetContent(b64.String())
		a.SetType(info.GetMime())
		a.SetFilename(info.GetName())
		a.SetDisposition("inline")
		a.SetContentID(f)

		email.AddAttachment(a)
	}

	request := sendgrid.GetRequest(
		handler.config.SendgridKey,
		"/v3/mail/send",
		"https://api.sendgrid.com",
	)
	request.Method = "POST"
	request.Body = mail.GetRequestBody(email)

	response, err := sendgrid.API(request)
	if err != nil {
		return err
	}

	if response.StatusCode/100 == 2 {
		return nil
	}

	return grpcerr.New(grpcerr.ProviderError, response.Body)
}

func (handler *handlerImpl) RetryPeriod() time.Duration {
	return time.Minute
}
