package postgres

import (
	"context"
	"database/sql"
	"errors"

	"github.com/jmoiron/sqlx"
	"github.com/lib/pq"
	"gitlab.mashroom.com/mashroom/backend/common/api/generic/query"
	"gitlab.mashroom.com/mashroom/backend/common/generic/psqlquery"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/model"
)

type repositoryNotification struct {
	DB           *sqlx.DB
	save         *sqlx.NamedStmt
	getByID      *sqlx.NamedStmt
	sync         *sqlx.NamedStmt
	removeUnused *sqlx.NamedStmt
	listCustom   *sqlx.NamedStmt
}

func prepareNotification(db *sqlx.DB) (stmt repositoryNotification, err error) {
	stmt.DB = db

	stmt.save, err = prepareNamed(db, queryNotificationSave)
	if err != nil {
		return
	}

	stmt.getByID, err = prepareNamed(db, queryNotificationGetByID)
	if err != nil {
		return
	}

	stmt.sync, err = prepareNamed(db, queryNotificationSync)
	if err != nil {
		return
	}

	stmt.removeUnused, err = prepareNamed(db, queryNotificationRemoveUnused)
	if err != nil {
		return
	}

	stmt.listCustom, err = prepareNamed(db, queryNotificationListCustom)
	if err != nil {
		return
	}

	return
}

const queryNotificationSave = `
insert into notifications(
  actor,
  entity,
  role,
  action,
  duration,
  channel,
  template,
  template_html,
  contacts,
  data,
  updated,
  created,
  disabled,
  customized
) values (
  :actor,
  :entity,
  :role,
  :action,
  :duration,
  :channel,
  :template,
  :template_html,
  :contacts,
  :data,
  now(),
  now(),
  :disabled,
  :customized
) on conflict(actor, entity, role, action, duration, channel) do update set
  template = :template,
  template_html = :template_html,
  contacts = :contacts,
  data = :data,
  updated = now(),
  disabled = :disabled,
  customized = :customized
returning
  id,
  actor,
  entity,
  role,
  action,
  duration,
  channel,
  template,
  template_html,
  contacts,
  data,
  updated,
  created,
  disabled,
  customized
`

func (repo *repositoryNotification) NotificationSave(ctx context.Context, m *model.Notification) error {
	err := repo.save.GetContext(ctx, m, m)

	return err
}

const queryNotificationGetByID = `
select
  id,
  actor,
  entity,
  role,
  action,
  duration,
  channel,
  template,
  template_html,
  contacts,
  data,
  updated,
  created,
  disabled,
  customized
from notifications where
  id = :id
`

func (repo *repositoryNotification) NotificationGetByID(ctx context.Context, id uint64) (*model.Notification, error) {
	m := &model.Notification{
		ID: id,
	}

	err := repo.getByID.GetContext(ctx, m, m)
	if err != nil {
		return nil, err
	}

	return m, nil
}

const queryNotificationSync = `
insert into notifications(
  actor,
  entity,
  role,
  action,
  duration,
  channel,
  template,
  template_html,
  contacts,
  data
) values (
  :actor,
  :entity,
  :role,
  :action,
  :duration,
  :channel,
  :template,
  :template_html,
  :contacts,
  :data
) on conflict(actor, entity, role, action, duration, channel) do update set
  template = (case when notifications.customized then notifications.template else :template end),
  template_html = (case when notifications.customized then notifications.template_html else :template_html end),
  contacts = (case when notifications.customized then notifications.contacts else :contacts end),
  data = (case when notifications.customized then notifications.data else :data end),
  updated = (case when notifications.customized then notifications.updated else now() end)
returning
  id,
  actor,
  entity,
  role,
  action,
  duration,
  channel,
  template,
  template_html,
  contacts,
  data,
  updated,
  created,
  disabled,
  customized
`

func (repo *repositoryNotification) NotificationSync(ctx context.Context, n *model.Notification) error {
	err := repo.sync.GetContext(ctx, n, n)

	return err
}

const queryNotificationRemoveUnused = `
delete from notifications where
  not (id = ANY(:used)) and 
  customized = false 
`

func (repo *repositoryNotification) NotificationRemoveUnused(ctx context.Context, used []uint64) error {
	var usedarr pq.Int64Array

	for _, u := range used {
		usedarr = append(usedarr, int64(u))
	}

	_, err := repo.removeUnused.ExecContext(ctx, struct {
		Used pq.Int64Array
	}{
		Used: usedarr,
	})

	return err
}

const queryNotificationListCustom = `
select
  id,
  actor,
  entity,
  role,
  action,
  duration,
  channel,
  template,
  template_html,
  contacts,
  data,
  updated,
  created,
  disabled,
  customized
from notifications where
  not (id = ANY(:used)) and 
  customized = true
`

func (repo *repositoryNotification) NotificationListCustom(
	ctx context.Context,
	known []uint64,
) (ms []*model.Notification, err error) {
	var usedarr pq.Int64Array

	for _, u := range known {
		usedarr = append(usedarr, int64(u))
	}

	err = repo.listCustom.SelectContext(ctx, &ms, struct {
		Used pq.Int64Array
	}{
		Used: usedarr,
	})

	if errors.Is(err, sql.ErrNoRows) {
		err = nil
	}

	return
}

func (repo *repositoryNotification) NotificationFilterList(
	ctx context.Context,
	filter *query.QueryParams,
) (ms []*model.Notification, count uint64, err error) {
	qq, args, err := psqlquery.BuildListQuery(model.Notification{}, "notifications", filter)
	if err != nil {
		return nil, 0, err
	}

	err = repo.DB.SelectContext(ctx, &ms, qq, args...)
	if err != nil {
		return nil, 0, err
	}

	qq, args, err = psqlquery.BuildCountQuery(model.Notification{}, "notifications", filter)
	if err != nil {
		return nil, 0, err
	}

	err = repo.DB.GetContext(ctx, &count, qq, args...)
	if err != nil {
		return nil, 0, err
	}

	return
}
