// Package service provides notification grpc service implementation
package service

import (
	"context"
	"fmt"
	"time"

	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/embed"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/handlers"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/model"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/task"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/template"
)

// Handler task keys
const (
	HandlerEmail    = "email"
	HandlerSMS      = "sms"
	HandlerPlatform = "platform"
)

// Handlers for tasks
type Handlers struct {
	Email    handlers.Handler
	SMS      handlers.Handler
	Platform handlers.Handler
}

// Config for service implementation
type Config struct {
	// Handlers map from handler code to it's instance
	Handlers Handlers
	// Persistent repository
	Repository model.Repository
	// Task queue repository
	Queue task.Repository
	// Interval between dequeueing for each of handlers
	DequeuePeriod time.Duration
	// Implementation of dequeue ticker, based on time.Ticker if nil
	DequeueTicker func(key string) <-chan time.Time
	// Embeds resolvers root
	Embeds *embed.Root
	// external gateway hostname
	Site string
	// ADMIN-targets support email
	SupportEmail string
	// Production mode
	Production bool
	// Notification templates
	Templates template.DescriptorMap
	// Sendgrid API key
	SendgridKey string
	// Sendgrid leads list ID
	SendgridListLeads string
}

// Service implementation
type Service interface {
	notification.NotificationServiceServer
}

func validateConfigHandlers(handlers *Handlers) error {
	if handlers.Email == nil {
		return fmt.Errorf("no email handler defined")
	}

	if handlers.SMS == nil {
		return fmt.Errorf("no sms handler defined")
	}

	if handlers.Platform == nil {
		return fmt.Errorf("no platform handler defined")
	}

	return nil
}

func validateConfig(config *Config) error {
	if config == nil {
		return fmt.Errorf("invalid service config")
	}

	err := validateConfigHandlers(&config.Handlers)
	if err != nil {
		return fmt.Errorf("%w in service config", err)
	}

	if config.Repository == nil {
		return fmt.Errorf("no repository in service config")
	}

	if config.Queue == nil {
		return fmt.Errorf("no task queue in service config")
	}

	if config.DequeuePeriod == 0 {
		return fmt.Errorf("zero dequeue period in service config")
	}

	if config.Embeds == nil {
		return fmt.Errorf("no embeds resolvers provided in service config")
	}

	if config.Site == "" {
		return fmt.Errorf("no site hostname in service config")
	}

	if config.Templates == nil {
		return fmt.Errorf("no templates in service config")
	}

	return nil
}

func (srv *serviceImpl) syncTemplateResolveBase(
	entity notification.Entity,
	action notification.Action,
	actor, role notification.Role,
	dur time.Duration,
) template.ChannelMap {
	actionmap := srv.config.Templates[entity]
	if actionmap == nil {
		actionmap = make(template.ActionMap)
		srv.config.Templates[entity] = actionmap
	}

	actormap := actionmap[action]
	if actormap == nil {
		actormap = make(template.ActorMap)
		actionmap[action] = actormap
	}

	rolemap := actormap[actor]
	if rolemap == nil {
		rolemap = make(template.RoleMap)
		actormap[actor] = rolemap
	}

	durmap := rolemap[role]
	if durmap == nil {
		durmap = make(template.DurationMap)
		rolemap[role] = durmap
	}

	chmap := durmap[dur]
	if chmap == nil {
		chmap = make(template.ChannelMap)
		durmap[dur] = chmap
	}

	return chmap
}

func (srv *serviceImpl) syncTemplatesKnownChanel(
	ctx context.Context,
	n *model.Notification,
	chmap template.ChannelMap,
) (id uint64, err error) {
	err = srv.config.Repository.NotificationSync(ctx, n)
	if err != nil {
		return
	}

	id = n.ID

	if n.Disabled {
		delete(chmap, n.Channel)
	}

	if n.Customized {
		chmap[n.Channel] = template.New(
			template.WithData(n.Data),
			template.WithGoTemplateSource(n.Template, n.TemplateHTML),
		)
	}

	return
}

func (srv *serviceImpl) syncTemplatesKnown(ctx context.Context) (known []uint64, err error) {
	for entity, actionmap := range srv.config.Templates {
		for action, actormap := range actionmap {
			for actor, rolemap := range actormap {
				for role, durmap := range rolemap {
					for dur, chmap := range durmap {
						for ch, tmpl := range chmap {
							var id uint64

							id, err = srv.syncTemplatesKnownChanel(ctx, &model.Notification{
								Actor:        actor,
								Entity:       entity,
								Role:         role,
								Action:       action,
								Duration:     dur,
								Channel:      ch,
								Template:     tmpl.Source(),
								TemplateHTML: tmpl.HTML(),
								Data:         tmpl.Data(),
								Contacts: model.Contacts{
									Email:    tmpl.Contacts().GetEmail(),
									Phone:    tmpl.Contacts().GetPhone(),
									Fullname: tmpl.Contacts().GetFullname(),
								},
							}, chmap)
							if err != nil {
								return
							}

							known = append(known, id)
						}
					}
				}
			}
		}
	}

	return
}

func (srv *serviceImpl) syncTemplates(ctx context.Context) error {
	known, err := srv.syncTemplatesKnown(ctx)
	if err != nil {
		return err
	}

	err = srv.config.Repository.NotificationRemoveUnused(ctx, known)
	if err != nil {
		return err
	}

	ns, err := srv.config.Repository.NotificationListCustom(ctx, known)
	if err != nil {
		return err
	}

	for _, n := range ns {
		chmap := srv.syncTemplateResolveBase(n.Entity, n.Action, n.Actor, n.Role, n.Duration)
		chmap[n.Channel] = template.New(
			template.WithData(n.Data),
			template.WithGoTemplateSource(n.Template, n.TemplateHTML),
			template.WithContacts(&notification.Contacts{
				Email:    n.Contacts.Email,
				Phone:    n.Contacts.Phone,
				Fullname: n.Contacts.Fullname,
			}),
		)
	}

	return nil
}

// NewService returns new service instance, initializing pending tasks and starting background dequeuers
func NewService(ctx context.Context, config *Config) (Service, error) {
	err := validateConfig(config)
	if err != nil {
		return nil, err
	}

	srv := &serviceImpl{
		config: *config,
	}

	if srv.config.DequeueTicker == nil {
		srv.config.DequeueTicker = func(key string) <-chan time.Time {
			return time.NewTicker(srv.config.DequeuePeriod).C
		}
	}

	err = srv.syncTemplates(ctx)
	if err != nil {
		return nil, err
	}

	go srv.handlerDequeue(ctx, HandlerEmail, config.Handlers.Email)

	go srv.handlerDequeue(ctx, HandlerSMS, config.Handlers.SMS)

	go srv.handlerDequeue(ctx, HandlerPlatform, config.Handlers.Platform)

	return srv, nil
}
