package service

import (
	"context"

	"gitlab.mashroom.com/mashroom/backend/common/generic/log"

	messaging "gitlab.mashroom.com/mashroom/backend/messaging/api"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/model"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/task"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/template"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/template/legacy"
)

func (srv *serviceImpl) SendEmail(
	ctx context.Context,
	req *notification.SendEmailRequest,
) (*notification.SendEmailResponse, error) {
	body := req.GetBody()

	t := task.NewEmail(
		task.WithEmailTo(req.GetTo(), ""),
		task.WithEmailSubject(req.GetSubject()),
		task.WithEmailBody(body, !req.GetTemplate(), req.GetHtml()),
		task.WithEmailReplyTo(req.GetReplyTo(), ""),
		task.WithEmailAttachments(req.GetAttachments()...),
	)

	err := srv.enqueueTask(ctx, &model.Log{
		Email:        req.GetTo(),
		EventChannel: notification.EventChannel_EVENT_CHANNEL_EMAIL,
	}, HandlerEmail, t)
	if err != nil {
		return nil, err
	}

	return &notification.SendEmailResponse{}, err
}

func (srv *serviceImpl) SendEmailV2(
	ctx context.Context,
	req *notification.SendEmailV2Request,
) (*notification.SendEmailResponse, error) {
	t := task.NewEmail(
		task.WithEmailTo(req.GetTo(), req.GetName()),
		task.WithEmailSubject(req.GetSubject()),
		task.WithEmailEmbeds(req.GetData()),
		task.WithEmailData(map[string]string{template.SendgridTemplateID: req.GetTemplate()}),
	)

	err := srv.enqueueTask(ctx, &model.Log{
		Email:        req.GetTo(),
		EventChannel: notification.EventChannel_EVENT_CHANNEL_EMAIL,
	}, HandlerEmail, t)
	if err != nil {
		return nil, err
	}

	return &notification.SendEmailResponse{}, err
}

func (srv *serviceImpl) SendSms(
	ctx context.Context,
	req *notification.SendSmsRequest,
) (*notification.SendSmsResponse, error) {
	t := task.NewSMS(req.GetTo(), req.GetBody())

	err := srv.enqueueTask(ctx, &model.Log{
		Phone:        req.GetTo(),
		EventChannel: notification.EventChannel_EVENT_CHANNEL_SMS,
	}, HandlerSMS, t)
	if err != nil {
		return nil, err
	}

	return &notification.SendSmsResponse{}, nil
}

func (srv *serviceImpl) SendPlatform(
	ctx context.Context,
	req *notification.SendPlatformRequest,
) (*notification.SendPlatformResponse, error) {
	var embeds []*notification.EventEmbed

	for _, e := range req.GetEmbeds() {
		embeds = append(embeds, &notification.EventEmbed{
			EmbedType:  messaging.EmbedType(e.GetEmbedType()),
			EmbedValue: e.GetEmbedValue(),
		})
	}

	t := task.NewPlatform(
		req.GetTo(),
		task.WithPlatformBody(req.GetBody()),
		task.WithPlatformEmbeds(embeds),
		task.WithPlatformEventType(req.GetEvent()),
	)

	err := srv.enqueueTask(ctx, &model.Log{
		Recipient:    req.GetTo(),
		EventChannel: notification.EventChannel_EVENT_CHANNEL_PLATFORM,
		EventEmbeds:  embeds,
	}, HandlerPlatform, t)
	if err != nil {
		return nil, err
	}

	return &notification.SendPlatformResponse{}, nil
}

func (srv *serviceImpl) SendEvent(
	ctx context.Context,
	req *notification.SendEventRequest,
) (*notification.SendEventResponse, error) {
	var userid, actorid string

	userid = req.GetUser()

	for _, e := range req.GetEmbeds() {
		if e.EmbedType == messaging.EmbedType_EMBED_TYPE_USER {
			actorid = e.GetEmbedValue()
			break
		}
	}

	var email, phone, fullname string

	for _, c := range req.GetContacts() {
		switch c.GetContactType() {
		case notification.ContactOfReceiverType_PHONE_CONTACT:
			phone = c.GetContactValue()
		case notification.ContactOfReceiverType_EMAIL_CONTACT:
			email = c.GetContactValue()
		case notification.ContactOfReceiverType_FULL_NAME_CONTACT:
			fullname = c.GetContactValue()
		}
	}

	nreq := legacy.TranslateLegacyNotification(req.GetEvent(), userid, actorid)
	nreq.Embeds = req.GetEmbeds()

	if len(nreq.GetTargets()) > 0 {
		nreq.GetTargets()[0].Contacts = &notification.Contacts{
			Email:    email,
			Phone:    phone,
			Fullname: fullname,
		}
	}

	log.Infof("legacy request translated: %v", nreq)

	_, err := srv.SendNotification(ctx, nreq)
	if err != nil {
		return nil, err
	}

	return &notification.SendEventResponse{}, nil
}
