package service

import (
	"context"
	"encoding/json"
	"strings"
	goTemplate "text/template"
	"time"

	"google.golang.org/protobuf/types/known/durationpb"

	"gitlab.mashroom.com/mashroom/backend/common/api/grpcerr"
	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	messaging "gitlab.mashroom.com/mashroom/backend/messaging/api"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/embed"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/model"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/task"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/template"
)

var errUnknownNotification = grpcerr.New(grpcerr.InvalidData, "unknown notification")

func (srv *serviceImpl) enqueueTask(
	ctx context.Context,
	log *model.Log,
	handler string,
	t *task.Task,
) error {
	err := srv.config.Repository.LogSave(ctx, log)
	if err != nil {
		return err
	}

	t.LogID = log.ID

	return srv.config.Queue.TaskEnqueue(ctx, handler, t)
}

func (srv *serviceImpl) resolveTemplateRoles(
	entity notification.Entity,
	action notification.Action,
	actor notification.Role,
) template.RoleMap {
	actions := srv.config.Templates[entity]
	if actions == nil {
		return nil
	}

	actors := actions[action]
	if actors == nil {
		return nil
	}

	roles := actors[actor]
	if roles == nil && actor == notification.Role_ROLE_UNKNOWN {
		for _, v := range actors {
			roles = v

			break
		}
	}

	return roles
}

func (srv *serviceImpl) resolveTemplateChannels(
	roles template.RoleMap,
	target notification.Role,
	dur time.Duration,
) template.ChannelMap {
	durs := roles[target]
	if durs == nil && target == notification.Role_ROLE_UNKNOWN {
		for _, v := range roles {
			durs = v
			break
		}
	}

	if durs == nil {
		return nil
	}

	return durs[dur]
}

func (srv *serviceImpl) resolveContacts(
	rcpt interface{},
	ch template.Template,
	t *notification.Target,
) (email, phone, name string) {
	email = embed.GetMapString(rcpt, "email")
	phone = embed.GetMapString(rcpt, "phone")
	name = embed.GetMapString(rcpt, "name")

	if email == "" && ch.Contacts().GetEmail() != "" {
		email = ch.Contacts().GetEmail()
	}

	if phone == "" && ch.Contacts().GetPhone() != "" {
		phone = ch.Contacts().GetPhone()
	}

	if name == "" && ch.Contacts().GetFullname() != "" {
		name = ch.Contacts().GetFullname()
	}

	if email == "" && t.GetRole() == notification.Role_ROLE_ADMIN {
		email = srv.config.SupportEmail
	}

	if phone == "" && t.GetContacts() != nil {
		phone = t.GetContacts().GetPhone()
	}

	if email == "" && t.GetContacts() != nil {
		email = t.GetContacts().GetEmail()
	}

	return
}

func (srv *serviceImpl) sendNotificationTarget(
	ctx context.Context,
	entity notification.Entity,
	action notification.Action,
	actor notification.Role,
	dur time.Duration,
	t *notification.Target,
	chans template.ChannelMap,
	embeds []*notification.EventEmbed,
	files []string,
) error {
	vals, rcpt, err := srv.config.Embeds.Resolve(ctx, t.GetUser(), t.GetContacts(), embeds)
	if err != nil {
		log.WithError(err).Error("resolving embeds", rcpt, vals)
	}

	for cht, ch := range chans {
		email, phone, name := srv.resolveContacts(rcpt, ch, t)

		buf := &strings.Builder{}

		err = ch.Execute(buf, vals)
		if err != nil {
			return err
		}

		switch cht {
		case notification.EventChannel_EVENT_CHANNEL_PLATFORM:
			ts := task.NewPlatform(
				t.GetUser(),
				task.WithPlatformBody(buf.String()),
				task.WithPlatformEmbeds(embeds),
				task.WithPlatformNotification(&notification.NotificationSpec{
					Entity:   entity,
					Action:   action,
					Actor:    actor,
					Target:   t.GetRole(),
					Duration: durationpb.New(dur),
				}),
				task.WithPlatformTitle(ch.Data()[template.Title]),
			)

			err = srv.enqueueTask(ctx, &model.Log{
				Recipient:    t.GetUser(),
				Email:        email,
				Phone:        phone,
				Actor:        actor,
				Entity:       entity,
				Role:         t.GetRole(),
				Action:       action,
				Duration:     dur,
				EventChannel: notification.EventChannel_EVENT_CHANNEL_PLATFORM,
				EventEmbeds:  embeds,
			}, HandlerPlatform, ts)
			if err != nil {
				return err
			}
		case notification.EventChannel_EVENT_CHANNEL_SMS:
			ts := task.NewSMS(phone, buf.String())

			err = srv.enqueueTask(ctx, &model.Log{
				Recipient:    t.GetUser(),
				Email:        email,
				Phone:        phone,
				Actor:        actor,
				Entity:       entity,
				Role:         t.GetRole(),
				Action:       action,
				Duration:     dur,
				EventChannel: notification.EventChannel_EVENT_CHANNEL_SMS,
				EventEmbeds:  embeds,
			}, HandlerSMS, ts)
			if err != nil {
				return err
			}
		case notification.EventChannel_EVENT_CHANNEL_EMAIL:
			var data []byte

			data, err = json.Marshal(vals)
			if err != nil {
				return err
			}

			var subjtmpl *goTemplate.Template

			subjtmpl, err = goTemplate.New("").Parse(ch.Data()[template.Subject])
			if err != nil {
				return err
			}

			subjbuf := &strings.Builder{}

			err = subjtmpl.Execute(subjbuf, vals)
			if err != nil {
				return err
			}

			ts := task.NewEmail(
				task.WithEmailTo(email, name),
				task.WithEmailSubject(subjbuf.String()),
				task.WithEmailBody(buf.String(), false, true),
				task.WithEmailEmbeds(data),
				task.WithEmailData(ch.Data()),
				task.WithEmailAttachments(files...),
			)

			err = srv.enqueueTask(ctx, &model.Log{
				Recipient:    t.GetUser(),
				Email:        email,
				Phone:        phone,
				Actor:        actor,
				Entity:       entity,
				Role:         t.GetRole(),
				Action:       action,
				Duration:     dur,
				EventChannel: notification.EventChannel_EVENT_CHANNEL_EMAIL,
				EventEmbeds:  embeds,
			}, HandlerEmail, ts)
			if err != nil {
				return err
			}
		}
	}

	return nil
}

func (srv *serviceImpl) SendNotification(
	ctx context.Context,
	req *notification.SendNotificationRequest,
) (*notification.SendNotificationResponse, error) {
	roles := srv.resolveTemplateRoles(req.GetEntity(), req.GetAction(), req.GetActor().GetRole())
	if roles == nil {
		return nil, errUnknownNotification
	}

	embeds := req.GetEmbeds()

	var userpresent bool

	var files []string

	for _, e := range embeds {
		if e.GetEmbedType() == messaging.EmbedType_EMBED_TYPE_USER {
			userpresent = true
		}

		if e.GetEmbedType() == messaging.EmbedType_EMBED_TYPE_FILE {
			files = append(files, e.GetEmbedValue())
		}
	}

	if !userpresent && req.GetActor().GetUser() != "" {
		embeds = append(embeds, &notification.EventEmbed{
			EmbedType:  messaging.EmbedType_EMBED_TYPE_USER,
			EmbedValue: req.GetActor().GetUser(),
		})
	}

	for _, t := range req.GetTargets() {
		if t.GetRole() == notification.Role_ROLE_ADMIN && !srv.config.Production {
			continue
		}

		dur := req.GetDuration().AsDuration()

		chans := srv.resolveTemplateChannels(roles, t.GetRole(), dur)
		if chans == nil {
			continue
		}

		err := srv.sendNotificationTarget(
			ctx,
			req.GetEntity(),
			req.GetAction(),
			req.GetActor().GetRole(),
			dur,
			t,
			chans,
			embeds,
			files,
		)
		if err != nil {
			return nil, err
		}
	}

	return &notification.SendNotificationResponse{}, nil
}

func (srv *serviceImpl) NotificationDefinitionList(
	ctx context.Context,
	req *notification.NotificationDefinitionListRequest,
) (*notification.NotificationDefinitionListResponse, error) {
	ns, total, err := srv.config.Repository.NotificationFilterList(ctx, req.GetParams())
	if err != nil {
		return nil, err
	}

	return &notification.NotificationDefinitionListResponse{
		Notifications: srv.renderNotifications(ns),
		Total:         int32(total),
	}, nil
}
