package service

import (
	"context"
	"encoding/json"
	"strconv"
	"strings"

	"github.com/sendgrid/rest"
	"github.com/sendgrid/sendgrid-go"
	"github.com/ttacon/libphonenumber"
	"gitlab.mashroom.com/mashroom/backend/common/api/grpcerr"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
)

type sendgridSubscribeListRecipient struct {
	ListIDs  []string                     `json:"list_ids"`
	Contacts []sendgridSubscribeRecipient `json:"contacts"`
}

type sendgridSubscribeRecipient struct {
	Email        string                 `json:"email"`
	FirstName    string                 `json:"first_name"`
	LastName     string                 `json:"last_name"`
	PhoneNumber  string                 `json:"phone_number"`
	CustomFields map[string]interface{} `json:"custom_fields"`
}

type sendgridSubscribeRecipientResponse struct {
	JobID string `json:"job_id"`
}

func (srv *serviceImpl) ListSubscribe(
	ctx context.Context,
	req *notification.ListSubscribeRequest,
) (*notification.ListSubscribeResponse, error) {
	if req.GetLeads() == nil {
		return nil, grpcerr.InvalidData
	}

	nameParts := strings.SplitN(req.GetLeads().GetName(), " ", 2)

	var firstName, lastName string

	switch len(nameParts) {
	case 1:
		firstName = strings.TrimSpace(nameParts[0])
	case 2:
		firstName, lastName = strings.TrimSpace(nameParts[0]), strings.TrimSpace(nameParts[1])
	default:
		return nil, grpcerr.New(grpcerr.InvalidData, "invalid name")
	}

	var phoneNumber string

	pn, err := libphonenumber.Parse(req.GetLeads().GetPhone(), libphonenumber.UNKNOWN_REGION)
	if err == nil {
		phoneNumber = libphonenumber.Format(pn, libphonenumber.E164)
	}

	receipts := sendgridSubscribeListRecipient{
		ListIDs: []string{srv.config.SendgridListLeads},
		Contacts: []sendgridSubscribeRecipient{
			{
				Email:       req.GetLeads().GetEmail(),
				FirstName:   firstName,
				LastName:    lastName,
				PhoneNumber: phoneNumber,
			},
		},
	}

	bs, err := json.Marshal(receipts)
	if err != nil {
		return nil, err
	}

	request := sendgrid.GetRequest(
		srv.config.SendgridKey,
		"/v3/marketing/contacts",
		"https://api.sendgrid.com",
	)
	request.Method = rest.Put
	request.Body = bs

	response, err := sendgrid.API(request)
	if err != nil {
		return nil, err
	}

	if response.StatusCode/100 != 2 {
		return nil, grpcerr.New(grpcerr.ProviderError, strconv.Itoa(response.StatusCode)+": "+response.Body)
	}

	var result sendgridSubscribeRecipientResponse

	err = json.Unmarshal(([]byte)(response.Body), &result)
	if err != nil {
		return nil, err
	}

	return &notification.ListSubscribeResponse{}, nil
}
