package task

import "encoding/json"

// EmailAddress email adress with optional name
type EmailAddress struct {
	Address string `json:"address,omitempty"`
	Name    string `json:"name,omitempty"`
}

// Email task
type Email struct {
	To          []*EmailAddress   `json:"to,omitempty"`
	ReplyTo     *EmailAddress     `json:"reply_to,omitempty"`
	Subject     string            `json:"subject,omitempty"`
	Body        string            `json:"body,omitempty"`
	Raw         bool              `json:"raw,omitempty"`
	HTML        bool              `json:"html,omitempty"`
	Embeds      json.RawMessage   `json:"embeds,omitempty"`
	Data        map[string]string `json:"data,omitempty"`
	Attachments []string          `json:"attachments,omitempty"`
}

// EmailTaskOption api mutator
type EmailTaskOption func(emailTask *Email)

// WithEmailTos returns EmailTaskOption providing multiple to destinations
func WithEmailTos(dest ...*EmailAddress) EmailTaskOption {
	return func(emailTask *Email) {
		emailTask.To = dest
	}
}

// WithEmailTo returns EmailTaskOption providing singular to destination
func WithEmailTo(address, name string) EmailTaskOption {
	return func(emailTask *Email) {
		emailTask.To = []*EmailAddress{
			{
				Address: address,
				Name:    name,
			},
		}
	}
}

// WithEmailReplyTo returns EmailTaskOption providing ReplyTo destination
func WithEmailReplyTo(address, name string) EmailTaskOption {
	return func(emailTask *Email) {
		emailTask.ReplyTo = &EmailAddress{
			Address: address,
			Name:    name,
		}
	}
}

// WithEmailSubject returns EmailTaskOption providing email subject
func WithEmailSubject(subj string) EmailTaskOption {
	return func(emailTask *Email) {
		emailTask.Subject = subj
	}
}

// WithEmailBody returns EmailTaskOption providing email body
func WithEmailBody(body string, raw, html bool) EmailTaskOption {
	return func(emailTask *Email) {
		emailTask.Body = body
		emailTask.Raw = raw
		emailTask.HTML = html
	}
}

// WithEmailEmbeds returns EmailTaskOption providing email template embeds
func WithEmailEmbeds(embeds json.RawMessage) EmailTaskOption {
	return func(emailTask *Email) {
		emailTask.Embeds = embeds
	}
}

// WithEmailData returns EmailTaskOption providing task data
func WithEmailData(data map[string]string) EmailTaskOption {
	return func(emailTask *Email) {
		emailTask.Data = data
	}
}

// WithEmailAttachments returns EmailTaskOption providing email attachment file IDs
func WithEmailAttachments(attachments ...string) EmailTaskOption {
	return func(emailTask *Email) {
		emailTask.Attachments = attachments
	}
}

// NewEmail returns new email task
func NewEmail(options ...EmailTaskOption) *Task {
	t := &Email{}

	for _, o := range options {
		o(t)
	}

	bs, _ := json.Marshal(t)

	return &Task{
		Data: bs,
	}
}
