package redis

import (
	"context"
	"encoding/json"
	"errors"
	"math"
	"time"

	"github.com/go-redis/redis/v8"
	"gitlab.mashroom.com/mashroom/backend/notification/internal/task"
)

type repositoryImpl struct {
	config Config
}

func (repo *repositoryImpl) TaskEnqueue(
	ctx context.Context,
	handler string,
	t *task.Task,
) error {
	streamkey := "notification:" + handler + ":stream"

	bs, err := json.Marshal(t)
	if err != nil {
		return err
	}

	err = repo.config.RedisClient.XAdd(ctx, &redis.XAddArgs{
		Stream: streamkey,
		Values: []string{"data", string(bs)},
	}).Err()

	return err
}

func (repo *repositoryImpl) dequeuePending(
	ctx context.Context,
	streamkey, streamgroup string,
	retry time.Duration,
) (t *task.Task, err error) {
	xpend, err := repo.config.RedisClient.XPendingExt(ctx, &redis.XPendingExtArgs{
		Stream:   streamkey,
		Group:    streamgroup,
		Start:    "-",
		End:      "+",
		Count:    math.MaxInt64,
		Consumer: "",
	}).Result()
	if err != nil {
		return
	}

	var pend *redis.XPendingExt

	for _, x := range xpend {
		if x.Idle > retry {
			pend = &x
			break
		}
	}

	if pend == nil {
		return
	}

	var msgs []redis.XMessage

	msgs, err = repo.config.RedisClient.XClaim(ctx, &redis.XClaimArgs{
		Stream:   streamkey,
		Group:    streamgroup,
		Consumer: repo.config.Consumer,
		MinIdle:  retry,
		Messages: []string{pend.ID},
	}).Result()
	if err != nil {
		return
	}

	if len(msgs) < 1 {
		return
	}

	data := msgs[0].Values["data"]

	s, ok := data.(string)
	if !ok {
		return
	}

	t = &task.Task{}

	err = json.Unmarshal(([]byte)(s), t)
	if err != nil {
		return nil, err
	}

	t.AckID = msgs[0].ID

	if t.Retry >= int(pend.RetryCount) {
		return
	}

	err = repo.config.RedisClient.XDel(ctx, streamkey, msgs[0].ID).Err()
	if err != nil {
		return nil, err
	}

	t = nil

	return
}

func (repo *repositoryImpl) dequeueNext(
	ctx context.Context,
	streamkey, streamgroup string,
) (t *task.Task, err error) {
	xs, err := repo.config.RedisClient.XReadGroup(ctx, &redis.XReadGroupArgs{
		Group:    streamgroup,
		Consumer: repo.config.Consumer,
		Streams:  []string{streamkey, ">"},
		Count:    1,
		Block:    -1,
		NoAck:    false,
	}).Result()
	if errors.Is(err, redis.Nil) {
		return nil, nil
	}

	if err != nil {
		return
	}

	if len(xs) < 1 {
		return
	}

	if len(xs[0].Messages) < 1 {
		return
	}

	data := xs[0].Messages[0].Values["data"]

	s, ok := data.(string)
	if !ok {
		return
	}

	t = &task.Task{}

	err = json.Unmarshal(([]byte)(s), t)
	if err != nil {
		return nil, err
	}

	t.AckID = xs[0].Messages[0].ID

	return
}

func (repo *repositoryImpl) TaskDequeue(
	ctx context.Context,
	handler string,
	retry time.Duration,
) (t *task.Task, err error) {
	defer func() {
		if errors.Is(err, redis.Nil) {
			err = nil
		}
	}()

	streamkey := "notification:" + handler + ":stream"

	t, err = repo.dequeuePending(ctx, streamkey, streamgroup, retry)
	if (err != nil && !errors.Is(err, redis.Nil)) || t != nil {
		return
	}

	t, err = repo.dequeueNext(ctx, streamkey, streamgroup)

	return
}

func (repo *repositoryImpl) TaskAck(ctx context.Context, handler string, ackid string) error {
	return repo.config.RedisClient.XAck(ctx, "notification:"+handler+":stream", streamgroup, ackid).Err()
}
