// Package template provides data types, builders and common constants for notification templates
package template

import (
	htmlTemplate "html/template"
	"io"
	"text/template"
	"time"

	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
)

// ChannelMap represents notification template associated with delivery channels
type ChannelMap map[notification.EventChannel]Template

// DurationMap represents ChannelMaps associated with notification periods
type DurationMap map[time.Duration]ChannelMap

// RoleMap represents DurationMaps associated with target roles
type RoleMap map[notification.Role]DurationMap

// ActorMap represents RoleMaps associated with actor roles
type ActorMap map[notification.Role]RoleMap

// ActionMap represents ActorMaps associated with entity actions
type ActionMap map[notification.Action]ActorMap

// DescriptorMap represents ActionMaps associated with entity types
type DescriptorMap map[notification.Entity]ActionMap

// Subject for email notifications
const Subject = "subject"

// SendgridTemplateID for sendgrid handler
const SendgridTemplateID = "sendgridID"

// Title for platform handler
const Title = "title"

type templateExecutor interface {
	Execute(wr io.Writer, data interface{}) error
}

// Template interface
type Template interface {
	Execute(wr io.Writer, data interface{}) error
	Data() map[string]string
	Source() string
	HTML() bool
	Contacts() *notification.Contacts
}

type templateImpl struct {
	executor templateExecutor
	source   string
	html     bool
	data     map[string]string
	contacts *notification.Contacts
}

func (t *templateImpl) Execute(wr io.Writer, data interface{}) error {
	if t.executor == nil {
		return nil
	}

	return t.executor.Execute(wr, data)
}

func (t *templateImpl) Source() string {
	return t.source
}

func (t *templateImpl) Data() map[string]string {
	return t.data
}

func (t *templateImpl) HTML() bool {
	return t.html
}

func (t *templateImpl) Contacts() *notification.Contacts {
	return t.contacts
}

// DataOption api mutator
type DataOption func(impl *templateImpl)

// WithSubject returns DataOption providing email subject
func WithSubject(subj string) DataOption {
	return func(impl *templateImpl) {
		impl.data[Subject] = subj
	}
}

// WithTitle returns DataOption providing platform title
func WithTitle(title string) DataOption {
	return func(impl *templateImpl) {
		impl.data[Title] = title
	}
}

// WithSendgridTemplate returns DataOption providing sendgrid template ID
func WithSendgridTemplate(id string) DataOption {
	return func(impl *templateImpl) {
		impl.data[SendgridTemplateID] = id
	}
}

// WithGoTemplate returns DataOption providing go template source
func WithGoTemplate(tmpl string) DataOption {
	return func(impl *templateImpl) {
		impl.executor = template.Must(template.New("").Parse(tmpl))
		impl.source = tmpl
	}
}

// WithContacts sets default contacts for notification
func WithContacts(contacts *notification.Contacts) DataOption {
	return func(impl *templateImpl) {
		impl.contacts = contacts
	}
}

// WithGoHTMLTemplate returns DataOption providing go template source (HTML-escaped)
func WithGoHTMLTemplate(tmpl string) DataOption {
	return func(impl *templateImpl) {
		impl.executor = htmlTemplate.Must(htmlTemplate.New("").Parse(tmpl))
		impl.source = tmpl
		impl.html = true
	}
}

// WithData returns DataOption providing data map
func WithData(data map[string]string) DataOption {
	return func(impl *templateImpl) {
		impl.data = data
	}
}

// WithGoTemplateSource returns DataOption providing go template source
func WithGoTemplateSource(src string, html bool) DataOption {
	if html {
		return WithGoHTMLTemplate(src)
	}

	return WithGoTemplate(src)
}

// New returns new template
func New(opts ...DataOption) Template {
	tmpl := &templateImpl{
		data: make(map[string]string),
	}

	for _, o := range opts {
		o(tmpl)
	}

	return tmpl
}

// ActorOption api mutator
type ActorOption func(m ActorMap)

// Action creates new entity action
func Action(options ...ActorOption) ActorMap {
	m := ActorMap{}

	for _, o := range options {
		o(m)
	}

	return m
}

// RoleOption api mutator
type RoleOption func(m RoleMap)

// WithActor returns ActorOption providing actor with a number of roles
func WithActor(actor notification.Role, options ...RoleOption) ActorOption {
	return func(m ActorMap) {
		rm, ok := m[actor]
		if !ok {
			rm = RoleMap{}
			m[actor] = rm
		}

		for _, o := range options {
			o(rm)
		}
	}
}

// WithRole returns RoleOption providing notifiction channels
func WithRole(role notification.Role, options ...ChannelOption) RoleOption {
	return func(m RoleMap) {
		cm, ok := m[role]
		if !ok {
			cm = DurationMap{}
			m[role] = cm
		}

		for _, o := range options {
			o(0, cm)
		}
	}
}

// WithActorRole returns ActorOption providing actor with singular role
func WithActorRole(actor, role notification.Role, options ...ChannelOption) ActorOption {
	return func(m ActorMap) {
		am, ok := m[actor]
		if !ok {
			am = RoleMap{}
			m[actor] = am
		}

		cm, ok := am[role]
		if !ok {
			cm = DurationMap{}
			am[role] = cm
		}

		for _, o := range options {
			o(0, cm)
		}
	}
}

// ChannelOption api mutator
type ChannelOption func(d time.Duration, durmap DurationMap)

// WithDuration returns ChannelOption wrapping underlying ChannelOptions with common duration
func WithDuration(dur time.Duration, opts ...ChannelOption) ChannelOption {
	return func(d time.Duration, durmap DurationMap) {
		for _, o := range opts {
			o(dur, durmap)
		}
	}
}

// WithChannel returns ChannelOption providing new notification channel
func WithChannel(channel notification.EventChannel, options ...DataOption) ChannelOption {
	return func(d time.Duration, durmap DurationMap) {
		tmpl := &templateImpl{
			data: make(map[string]string),
		}

		for _, o := range options {
			o(tmpl)
		}

		chmap, ok := durmap[d]
		if !ok {
			chmap = ChannelMap{}
			durmap[d] = chmap
		}

		chmap[channel] = tmpl
	}
}
