// Package legacy provides translations from legacy notifications to current format
package legacy

import (
	"time"

	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	"google.golang.org/protobuf/types/known/durationpb"
)

// TranslateLegacyNotification takes recipient and actor and tries to look up a translation to current notification
// format
func TranslateLegacyNotification(
	n notification.EventType,
	userid, actorid string,
) *notification.SendNotificationRequest {
	t, ok := translations[n]
	if !ok {
		return nil
	}

	return t(userid, actorid)
}

type resolver func(userid, actorid string) *notification.SendNotificationRequest

func makeRequest(
	actor notification.Role,
	entity notification.Entity,
	action notification.Action,
	role notification.Role,
	duration time.Duration,
) resolver {
	var dur *durationpb.Duration

	if duration > 0 {
		dur = durationpb.New(duration)
	}

	return func(userid, actorid string) *notification.SendNotificationRequest {
		return &notification.SendNotificationRequest{
			Actor: &notification.Actor{
				User: actorid,
				Role: actor,
			},
			Entity: entity,
			Action: action,
			Targets: []*notification.Target{
				{
					User: userid,
					Role: role,
				},
			},
			Duration: dur,
		}
	}
}

var translations = map[notification.EventType]resolver{
	notification.EventType_EVENT_AUTH_SIGN_UP: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_USER,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_AUTHOR,
		0,
	),
	notification.EventType_EVENT_INVITATION_CREATE_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_INVITATION,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_INVITATION_CREATE_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_INVITATION,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_LISTING_INVITATION_REJECTED_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_INVITATION,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_LISTING_INVITATION_REJECTED_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_INVITATION,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_INVITATION_CREATE_MANAGER_TO_LANDLORD: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_INVITATION_CREATE: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_TEAMUP_TENANT_INVITES_LANDLORD_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_TEAMUP_LANDLORD_ACCEPTS_VIEWING_HOST_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_TEAMUP_LANDLORD_ACCEPTS_VIEWING_HOST_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_TEAMUP_TENANT_ACCEPTS_VIEWING_HOST_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_TEAMUP_TENANT_ACCEPTS_VIEWING_HOST_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_TEAMUP_LANDLORD_DECLINES_VIEWING_HOST_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_TEAMUP_LANDLORD_DECLINES_VIEWING_HOST_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_TEAMUP_TENANT_DECLINES_VIEWING_HOST_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_TEAMUP_LANDLORD_END_VIEWING_HOST_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_CANCEL,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_TEAMUP_LANDLORD_END_VIEWING_HOST_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_CANCEL,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_TEAMUP_TENANT_REQUESTS_END_VIEWING_HOST_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_CANCEL,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_TEAMUP_TENANT_REQUESTS_END_VIEWING_HOST_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_CANCEL,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_TEAMUP_TENANT_INVITES_24HOUR_REMINDER_LANDLORD: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_LANDLORD,
		time.Hour*24,
	),
	notification.EventType_EVENT_TEAMUP_TENANT_ACCEPTS_VIEWING_HOST_BEFORE_LISTED_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER_UNLISTED,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_PROPERTY_MANAGE_VIEWING_BY_LANDLORD_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_LISTING,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_PROPERTY_MANAGE_VIEWING_BY_TENANT_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_LISTING,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_MANAGER,
		0,
	),
	notification.EventType_EVENT_LISTING_AWAITING_VERIFICATION_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_LISTING,
		notification.Action_ACTION_PENDING,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_PROPERTY_INCOMPLETE_SIX_HOUR_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_LISTING,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_LANDLORD,
		time.Hour*6,
	),
	notification.EventType_EVENT_PROPERTY_INCOMPLETE_ONE_DAY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_LISTING,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_LANDLORD,
		time.Hour*24,
	),
	notification.EventType_EVENT_PROPERTY_INCOMPLETE_THREE_DAYS_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_LISTING,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_LANDLORD,
		time.Hour*72,
	),
	notification.EventType_EVENT_MAINTENANCE_TENANT_CREATES_ISSUE_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_ISSUE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_MAINTENANCE_TENANT_CREATES_ISSUE_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_ISSUE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_MAINTENANCE_LANDLORD_CREATES_ISSUE_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_PROPERTY_ISSUE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_MAINTENANCE_LANDLORD_CREATES_ISSUE_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_PROPERTY_ISSUE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_MAINTENANCE_LANDLORD_ASSIGNS_LOCALHEROES_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_ISSUE_ASSIGN_LOCALHEROES,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_MAINTENANCE_LANDLORD_ASSIGNS_LOCALHEROES_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_ISSUE_ASSIGN_LOCALHEROES,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_MAINTENANCE_LANDLORD_ASSIGNS_CONTRACTOR_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_ISSUE_ASSIGN_CONTRACTOR,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_MAINTENANCE_LANDLORD_ASSIGNS_CONTRACTOR_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_ISSUE_ASSIGN_CONTRACTOR,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_MAINTENANCE_UPDATE_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_ISSUE,
		notification.Action_ACTION_UPDATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_MAINTENANCE_UPDATE_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_ISSUE,
		notification.Action_ACTION_UPDATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_MAINTENANCE_RESOLVED: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_ISSUE,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_MAINTENANCE_TASK_NOTIFY_USER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TASK,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_MESSAGE_RECEIVED: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_MESSAGE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_UNKNOWN,
		0,
	),
	notification.EventType_EVENT_OFFER_REQUEST_MANAGER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_SENT_CONFIRM_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_UPDATE_MANAGER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_UPDATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_ACCEPT_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_JOURNEY_COMPLETE_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_PAYMENT_READY_MOVEIN_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_REJECTED_CONFIRM_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_REJECTED_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_ACCEPTED_WITHDRAWN_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_CANCEL,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_ACCEPTED_WITHDRAWN_CONFIRM_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_CANCEL,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_ACCEPTED_CONFIRM_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_LANDLORD,
		0,
	),

	notification.EventType_EVENT_OFFER_RECEIVED_NOT_RESPONDED_REMIND_LANDLORD_3HRS: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_PENDING,
		notification.Role_ROLE_LANDLORD,
		time.Hour*3,
	),
	notification.EventType_EVENT_OFFER_RECEIVED_NOT_RESPONDED_REMIND_LANDLORD_1WEEK_LEFT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_LANDLORD,
		time.Hour*24*7,
	),
	notification.EventType_EVENT_OFFER_RECEIVED_NOT_RESPONDED_REMIND_TENANT_1WEEK_LEFT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_TENANT,
		time.Hour*24*7,
	),
	notification.EventType_EVENT_HOLDING_DEPOSIT_12HOURS_REMIND_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_HOLDING_DEPOSIT_PAYMENT,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_TENANT,
		time.Hour*12,
	),
	notification.EventType_EVENT_OFFER_HOLDING_DEPOSIT_CONFIRM_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_HOLDING_DEPOSIT_PAYMENT,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_HOLDING_DEPOSIT_CONFIRM_MANAGER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_HOLDING_DEPOSIT_PAYMENT,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_LANDLORD_ACCEPTED_OTHER_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_OTHER,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_VERIFICATION_SUCCESS_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_REFERENCE_CHECKS,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_VERIFICATION_SUCCESS_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_REFERENCE_CHECKS,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_HOLDING_DEPOSIT_NOT_PAID_MANAGER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_HOLDING_DEPOSIT_PAYMENT,
		notification.Action_ACTION_EXPIRE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_HOLDING_DEPOSIT_EXPIRED_24HOURS_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_HOLDING_DEPOSIT_PAYMENT,
		notification.Action_ACTION_EXPIRE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_HOLDING_DEPOSIT_NOT_PAID_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_HOLDING_DEPOSIT_PAYMENT,
		notification.Action_ACTION_EXPIRE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_PAYMENT_PAID_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_RENT_PAYMENT,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_PAYMENT_PAID_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_RENT_PAYMENT,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_PAYMENT_PAID_EXTRA_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_EXTRA_PAYMENT,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_PAYMENT_PAID_EXTRA_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_EXTRA_PAYMENT,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_PAYMENT_MASHROOM_INVOICE: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PAYMENT_INVOICE,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_UNKNOWN,
		0,
	),
	notification.EventType_EVENT_PROPERTY_PUBLISHED_MANAGER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_PUBLICATION,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_MANAGER,
		0,
	),
	notification.EventType_EVENT_SIGNING_READY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_SIGNATURE,
		notification.Action_ACTION_PENDING,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_SIGNING_READY_MANAGER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_SIGNATURE,
		notification.Action_ACTION_PENDING,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_SIGNING_SIGNED_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_SIGNATURE,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_SIGNING_SIGNED_MANAGER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_SIGNATURE,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_VIEWING_REQUEST_MANAGER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_MANAGER,
		0,
	),
	notification.EventType_EVENT_VIEWING_REQUEST_MANAGER_REMIND_3HOURS: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_PENDING,
		notification.Role_ROLE_MANAGER,
		time.Hour*3,
	),
	notification.EventType_EVENT_VIEWING_REQUEST_MANAGER_REMIND_12HOURS: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_PENDING,
		notification.Role_ROLE_MANAGER,
		time.Hour*12,
	),
	notification.EventType_EVENT_VIEWING_REQUEST_TENANT_FAV: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING_FAVORITE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_VIEWING_ACCEPT_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_VIEWING_ACCEPTED_REMIND_TENANT_1DAY: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_TENANT,
		time.Hour*24,
	),
	notification.EventType_EVENT_VIEWING_ACCEPTED_REMIND_LANDLORD_1DAY: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_MANAGER,
		time.Hour*24,
	),
	notification.EventType_EVENT_VIEWING_ACCEPTED_REMIND_TENANT_12HOUR: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_TENANT,
		time.Hour*12,
	),
	notification.EventType_EVENT_VIEWING_TODAY_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_VIEWING_TODAY_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_MANAGER,
		0,
	),
	notification.EventType_EVENT_VIEWING_ACCEPTED_REMIND_LANDLORD_12HOUR: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_MANAGER,
		time.Hour*12,
	),
	notification.EventType_EVENT_VIEWING_ACCEPTED_REMIND_2DAY: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_MANAGER,
		time.Hour*48,
	),
	notification.EventType_EVENT_VIEWING_DECLINE_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_VIEWING_RESCHEDULE_REQUEST_TENANT: makeRequest(
		notification.Role_ROLE_MANAGER,
		notification.Entity_ENTITY_PROPERTY_VIEWING_RESCHEDULE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_VIEWING_RESCHEDULE_REQUEST_MANAGER_CONFIRM: makeRequest(
		notification.Role_ROLE_MANAGER,
		notification.Entity_ENTITY_PROPERTY_VIEWING_RESCHEDULE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_MANAGER,
		0,
	),
	notification.EventType_EVENT_VIEWING_RESCHEDULE_ACCEPT_MANAGER: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_VIEWING_RESCHEDULE,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_MANAGER,
		0,
	),
	notification.EventType_EVENT_VIEWING_RESCHEDULE_ACCEPTED_LANDLORD: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_VIEWING_RESCHEDULE,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_MANAGER,
		0,
	),
	notification.EventType_EVENT_VIEWING_RESCHEDULE_REQUEST_MANAGER: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_VIEWING_RESCHEDULE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_MANAGER,
		0,
	),
	notification.EventType_EVENT_VIEWING_RESCHEDULE_REQUEST_TENANT_CONFIRM: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_VIEWING_RESCHEDULE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_VIEWING_RESCHEDULE_ACCEPT_TENANT: makeRequest(
		notification.Role_ROLE_MANAGER,
		notification.Entity_ENTITY_PROPERTY_VIEWING_RESCHEDULE,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_VIEWING_COMPLETE_MANAGER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_VIEWING_COMPLETE_MANAGER_20MINS: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_TENANT,
		time.Minute*20,
	),
	notification.EventType_EVENT_VIEWING_REQUEST_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_VIEWING_REQUEST_MANAGER_CANCEL: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_CANCEL,
		notification.Role_ROLE_MANAGER,
		0,
	),
	notification.EventType_EVENT_VIEWING_REQUEST_MANAGER_EXPIRED: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_EXPIRE,
		notification.Role_ROLE_MANAGER,
		0,
	),
	notification.EventType_EVENT_VIEWING_REQUEST_TENANT_EXPIRED: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING,
		notification.Action_ACTION_EXPIRE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_VIEWING_REWARD_PAID: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING_REWARD,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_MANAGER,
		0,
	),
	notification.EventType_EVENT_PROPERTY_WITHDRAWN: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_PUBLICATION,
		notification.Action_ACTION_CANCEL,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_PAYMENT_PAID_HEI_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_INSURANCE_HEI_PAYMENT,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_PAYMENT_NEW_UPLOADED: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PAYMENT_TRANSACTION,
		notification.Action_ACTION_UPDATE,
		notification.Role_ROLE_ADMIN,
		0,
	),
	notification.EventType_EVENT_PORTAL_RESPONSE: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_QUESTION,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_SERVICE_PAID_ADMIN: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_EXTRA_PAYMENT,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_ADMIN,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_END_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_END_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_CLAIM_END_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_CLAIM_END_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_NEAR_END_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_NEAR_END_NOTIFY_LANDLORD_REMINDER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_LANDLORD,
		time.Hour*24*60,
	),
	notification.EventType_EVENT_OFFER_TENANCY_OFFERED_RENEWAL_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_RENEWAL,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TENANT_TERMINATE_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_TENANCY_TERMINATION,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TENANT_TERMINATE_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_PROPERTY_TENANCY_TERMINATION,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_CONFIRMED_TERMINATE_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_PROPERTY_TENANCY_TERMINATION,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_LANDLORD_TERMINATE_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_PROPERTY_TENANCY_TERMINATION,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_ENDED_DEPOSIT_REMINDER_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_DEPOSIT,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_LANDLORD,
		time.Hour*24*10,
	),
	notification.EventType_EVENT_OFFER_TENANCY_ENDED_DEPOSIT_REMINDER_3DAYS_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_DEPOSIT,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_LANDLORD,
		time.Hour*24*3,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_RAISED_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_RAISED_CONFIRM_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_DISPUTED_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM_DISPUTE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_DISPUTED_1DAY_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM_DISPUTE,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_TENANT,
		time.Hour*24,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_DISPUTED_2DAY_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM_DISPUTE,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_TENANT,
		time.Hour*48,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_DISPUTED_LAST_CHANCE_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM_DISPUTE,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_TENANT,
		time.Hour*24*5,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_DISPUTED_LAST_CHANCE_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM_DISPUTE,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_LANDLORD,
		time.Hour*24*5,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_DISPUTED_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM_DISPUTE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_AGREED_3DAYS_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM_PAYMENT,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_TENANT,
		time.Hour*24*3,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_AGREED_7DAYS_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM_PAYMENT,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_TENANT,
		time.Hour*24*7,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_AGREE_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_PAID_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM_PAYMENT,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_NO_CLAIMS_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_CLAIM,
		notification.Action_ACTION_EXPIRE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_DEPOSIT_RELEASED_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_TENANCY_DEPOSIT,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_VIEWING_BROADCASTING_STARTED: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING_BROADCASTING,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_ADMIN_UPLOADED_DOC_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_DOCUMENT,
		notification.Action_ACTION_UPDATE,
		notification.Role_ROLE_UNKNOWN,
		0,
	),
	notification.EventType_EVENT_PAYMENT_INACTIVE_MANDATE: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PAYMENT_MANDATE,
		notification.Action_ACTION_EXPIRE,
		notification.Role_ROLE_ADMIN,
		0,
	),
	notification.EventType_EVENT_PASSWORD_RESET: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_USER_PASSWORD,
		notification.Action_ACTION_EXPIRE,
		notification.Role_ROLE_AUTHOR,
		0,
	),
	notification.EventType_EVENT_LESS_THAN_3_VIEWINGS_IN_A_WEEK_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_PROPERTY_VIEWING_RARITY,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_MORTGAGE_REENGAGE_REMINDER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_MORTGAGE_NOTE,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_ADMIN,
		0,
	),
	notification.EventType_EVENT_ADVANCED_RENT_REQUEST_ACCEPTED_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_RENT_ADVANCE,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_ADVANCED_RENT_REQUEST_ACCEPTED_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_RENT_ADVANCE,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_ADVANCED_RENT_REQUEST_DENIED_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_RENT_ADVANCE,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_ADVANCED_RENT_REQUEST_CREATED_NOTIFY_ADMIN: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_RENT_ADVANCE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_ADMIN,
		0,
	),
	notification.EventType_EVENT_ADVANCED_RENT_REQUEST_CANCELLED_NOTIFY_ADMIN: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_RENT_ADVANCE,
		notification.Action_ACTION_CANCEL,
		notification.Role_ROLE_ADMIN,
		0,
	),
	notification.EventType_EVENT_ADVANCED_RENT_PAYMENT_CONFIRMED_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_RENT_ADVANCE,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_ADVANCED_RENT_PAYMENT_CONFIRMED_NOTIFY_ADMIN: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_RENT_ADVANCE,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_ADMIN,
		0,
	),
	notification.EventType_EVENT_CONTRACT_CLAUSE_DELETED_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_CLAUSE,
		notification.Action_ACTION_REMOVE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_CONTRACT_CLAUSE_EDITED_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_CLAUSE,
		notification.Action_ACTION_UPDATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_CONTRACT_CLAUSE_CREATED_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_CLAUSE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_CONTRACT_LANDLORD_CLAUSE_EDITED_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_CLAUSE,
		notification.Action_ACTION_UPDATE,
		notification.Role_ROLE_ADMIN,
		0,
	),
	notification.EventType_EVENT_CONTRACT_LANDLORD_CLAUSE_CREATED_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_CLAUSE,
		notification.Action_ACTION_CREATE,
		notification.Role_ROLE_ADMIN,
		0,
	),
	notification.EventType_EVENT_CONTRACT_CLAUSE_ACCEPT_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_CLAUSE,
		notification.Action_ACTION_ACCEPT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_CONTRACT_CLAUSE_REJECT_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_CLAUSE,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_CONTRACT_LANDLORD_PROCEEDED_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_OFFER_SIGNATURE_INVITATION,
		notification.Action_ACTION_PENDING,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_CONTRACT_LANDLORD_PROCEEDED_REMIND_TENANT_2HOURS: makeRequest(
		notification.Role_ROLE_LANDLORD,
		notification.Entity_ENTITY_OFFER_SIGNATURE_INVITATION,
		notification.Action_ACTION_PENDING,
		notification.Role_ROLE_TENANT,
		time.Hour*2,
	),
	notification.EventType_EVENT_CONTRACT_TENANT_PROCEEDED_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_OFFER_SIGNATURE_INVITATION,
		notification.Action_ACTION_PENDING,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_CONTRACT_TENANT_PROCEEDED_REMIND_LANDLORD_2HOURS: makeRequest(
		notification.Role_ROLE_TENANT,
		notification.Entity_ENTITY_OFFER_SIGNATURE_INVITATION,
		notification.Action_ACTION_PENDING,
		notification.Role_ROLE_LANDLORD,
		time.Hour*2,
	),
	notification.EventType_EVENT_HOLDING_DEPOSIT_TOKENIO_CONFIRM_PAYMENT_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_HOLDING_DEPOSIT_PAYMENT,
		notification.Action_ACTION_COMPLETE,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_HOLDING_DEPOSIT_BANK_VERIFYING_PAYMENT_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_HOLDING_DEPOSIT_PAYMENT,
		notification.Action_ACTION_VERIFY,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_HOLDING_DEPOSIT_BANK_CONFIRM_PAYMENT_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_HOLDING_DEPOSIT_PAYMENT,
		notification.Action_ACTION_VERIFY,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_HOLDING_DEPOSIT_TENANT_WITHDRAWN_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_DEPOSITED,
		notification.Action_ACTION_CANCEL,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_HOLDING_DEPOSIT_TENANT_WITHDRAWN_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_DEPOSITED,
		notification.Action_ACTION_CANCEL,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_HOLDING_DEPOSIT_LANDLORD_WITHDRAWN_NOTIFY_LANDLORD: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_DEPOSITED,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_LANDLORD,
		0,
	),
	notification.EventType_EVENT_HOLDING_DEPOSIT_LANDLORD_WITHDRAWN_NOTIFY_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_DEPOSITED,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_DENIED_AFTER_DEPOSIT_TENANT: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_DEPOSITED,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_TENANT,
		0,
	),
	notification.EventType_EVENT_OFFER_DENIED_AFTER_DEPOSIT_ADMIN: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_OFFER_DEPOSITED,
		notification.Action_ACTION_REJECT,
		notification.Role_ROLE_ADMIN,
		0,
	),
	notification.EventType_EVENT_DOCUMENT_EXPIRES_REMINDER: makeRequest(
		notification.Role_ROLE_UNKNOWN,
		notification.Entity_ENTITY_DOCUMENT_EXPIRATION,
		notification.Action_ACTION_REMIND,
		notification.Role_ROLE_UNKNOWN,
		0,
	),
	// notification.EventType_EVENT_OFFER_TENANCY_NEAR_END_NOTIFY_TENANT: missing,
	// notification.EventType_EVENT_INVITATION_CONTACT_VALIDATION: unused,
	// notification.EventType_EVENT_INVITATION_REJECTED: unused,
	// notification.EventType_EVENT_INVITATION_IGNORED:  unused,
	// notification.EventType_EVENT_INVITATION_ACCEPTED: unused,
	// notification.EventType_EVENT_INVITATION_CONTACT_SIGNING: unused,
	// notification.EventType_EVENT_INVITATION_TO_TENANT_NO_RESPONSE_ONE_DAY: unused,
	// notification.EventType_EVENT_INVITATION_TO_TENANT_NO_RESPONSE_THREE_DAYS: unused,
	// notification.EventType_EVENT_INVITATION_TO_LANDLORD_NO_RESPONSE_ONE_DAY: unused,
	// notification.EventType_EVENT_INVITATION_TO_LANDLORD_NO_RESPONSE_THREE_DAYS: unused,
	// notification.EventType_EVENT_LISTING_INPROCESS: unused,
	// notification.EventType_EVENT_LISTING_AWAITINGPUBLICATION: unused,
	// notification.EventType_EVENT_LISTING_NOTAUTHORIZED: unused,
	// notification.EventType_EVENT_LISTING_ERRORS: unused,
	// notification.EventType_EVENT_LISTING_DENIED: unused,
	// notification.EventType_EVENT_LISTING_LISTED: unused,
	// notification.EventType_EVENT_LISTING_ALREADYLISTED: unused,
	// notification.EventType_EVENT_LISTING_ACCOUNTERRORS: unused,
	// notification.EventType_EVENT_LISTING_SUCCESS: unused,
	// notification.EventType_EVENT_LISTING_AWAITINGLANDLORD: unused,
	// notification.EventType_EVENT_LISTING_NOLANDLORD: unused,
	// notification.EventType_EVENT_LISTING_NEEDPAY: unused,
	// notification.EventType_EVENT_LISTING_NEEDDOCUMENT: unused,
	// notification.EventType_EVENT_LISTING_LISTED_TENANT: unused,
	// notification.EventType_EVENT_LISTING_EXPIRY_DATE_THREE_DAYS_LANDLORD: unused,
	// notification.EventType_EVENT_LISTING_EXPIRY_DATE_THREE_DAYS_ADMIN: unused,
	// notification.EventType_EVENT_LISTING_EXPIRY_DATE_ZERO_DAYS_ADMIN: unused,
	// notification.EventType_EVENT_MAINTENANCE_CREATE_TENANT: unused
	// notification.EventType_EVENT_MAINTENANCE_CREATE_LANDLORD: unused
	// notification.EventType_EVENT_NEW_MESSAGE_RECEIVED: unused,
	// notification.EventType_EVENT_OFFER_CANCEL_MANAGER: unused,
	// notification.EventType_EVENT_OFFER_DECLINE_TENANT: unused,
	// notification.EventType_EVENT_OFFER_SUBMITTED: unused,
	// notification.EventType_EVENT_OFFER_SUBMITTED_LANDLORD: unused,
	// notification.EventType_EVENT_OFFER_WITHDRAWN_LANDLORD: unused,
	// notification.EventType_EVENT_OFFER_VERIFICATION_LANDLORD: unused,
	// notification.EventType_EVENT_OFFER_VERIFICATION_TENANT: unused,
	// notification.EventType_EVENT_OFFER_GUARANTOR_REQUIRED_TENANT: unused,
	// notification.EventType_EVENT_PAYMENT_MISSING: unused,
	// notification.EventType_EVENT_PAYMENT_PENDING: unused,
	// notification.EventType_EVENT_PROPERTY_LINK_TENANT: unused,
	// notification.EventType_EVENT_PROPERTY_UNPUBLISHED_MANAGER: unused,
	// notification.EventType_EVENT_VIEWING_RESCHEDULE_ACCEPT_LANDLORD: unused,
	// notification.EventType_EVENT_VIEWING_COMPLETE_REMIND_12HOUR: unused,
	// notification.EventType_EVENT_VIEWING_COMPLETE_REMIND_1DAY: unused,
	// notification.EventType_EVENT_VIEWING_COMPLETE_TENANT_FAV: unused,
	// notification.EventType_EVENT_VIEWING_HOST_CHANGED: unused,
	// notification.EventType_EVENT_PAYMENT_PAID_RGI_LANDLORD: unused,
	// notification.EventType_EVENT_PROPERTY_INCOMPLETE_ONE_WEEK_LANDLORD: unused,
	// notification.EventType_EVENT_OFFER_TENANCY_ENDED_NOTIFY_LANDLORD: unused,
	// notification.EventType_EVENT_OFFER_TENANCY_ENDED_DEPOSIT_RELEASED_NOTIFY_TENANT: unused,
	// notification.EventType_EVENT_OFFER_TENANCY_TERMINATE_CLAIM_AGREED_NOTIFY_TENANT: unused,
	// notification.EventType_EVENT_OFFER_TENANT_UPDATED_CONFIRM_TENANT: unused,
	// notification.EventType_EVENT_HOLDING_DEPOSIT_EXPIRED_NOTIFY_LANDLORD: unused,
}
